package com.iver.templateLayout;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.View;

import com.iver.cit.gvsig.Print;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameText;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameView;

import java.awt.geom.Rectangle2D;

import java.util.ArrayList;
import java.util.Hashtable;


/**
 * Extensin de ejemplo que genera una ficha por cada shape seleccionado,
 * cambiando los tag por su valor en la tabla.
 *
 * @author Vicente Caballero Navarro
 */
public class TemplateExtension implements Extension {
	//Indica si es la primera ficha lanzada a imprimir.
	private boolean isFirst = true;

	//Layout que modificamos para la creacin de las fichas.
	private Layout layout = null;

	//Array de booleanos que contiene en la posicin:
	//"0"-->Si se selecciona la impresin de todas las fichas generadas se pone a true.
	//"1"-->Si se selecciona cancelar de todas las fichas generadas se pone a false.
	private boolean[] printAll = { false, true };

	//Otra extensin, que es utlizada para lanzar el Layout a impresin.
	private Print printExtension = new Print();

	//Almacena el texto de todos los FFrameText del Layout.
	private Hashtable hashtext = new Hashtable();

	//ArrayList que almacena todos los FFrames que contiene el Layout.
	private ArrayList fframes = null;

	/**
	 * @see com.iver.mdiApp.plugins.Extension#isEnabled()
	 */
	public boolean isEnabled() {
		//Devuelvo true, y de esta forma siempre estarn activos.
		return true;
	}

	/**
	 * @see com.iver.mdiApp.plugins.Extension#isVisible()
	 */
	public boolean isVisible() {
		//Se obtiene la vista activada.
		View f = PluginServices.getMDIManager().getActiveView();

		//Se comprueba si la vista es null y se devuelve un false para no hacer visible la extensin.
		if (f == null) {
			return false;
		}

		//Se comprueba si la vista activa contiene un Layout y 
		//si es as se devuelve true como visible o false si no contiene un Layout.
		if (f.getClass() == Layout.class) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Guarda el contenido de todos los FFrameText para poder  recuperarlos al
	 * final.
	 */
	private void initFFrameText() {
		//Recorro todos los FFrames que contiene el Layout y compruebo que son una instancia de FFrameText, 
		//para guardar en hastext el texto y su FFrameText.
		for (int ft = 0; ft < fframes.size(); ft++) {
			if (fframes.get(ft) instanceof FFrameText) {
				ArrayList al = new ArrayList();

				for (int i = 0;
						i < ((FFrameText) fframes.get(ft)).getText().size();
						i++) {
					al.add(((FFrameText) fframes.get(ft)).getText().get(i));
				}

				hashtext.put(fframes.get(ft), al);
			}
		}
	}

	/**
	 * Recupera el texto que inicialmente tenan todos FFrameText.
	 */
	private void endFFrameText() {
		//Recorro todos los FFrames que contiene el Layout y compruebo que son una instancia de FFrameText, 
		//para inicializarlo con su texto original almacenado en hashtext.
		for (int ft = 0; ft < fframes.size(); ft++) {
			if (fframes.get(ft) instanceof FFrameText) {
				ArrayList text = (ArrayList) hashtext.get(fframes.get(ft));
				((FFrameText) fframes.get(ft)).getText().clear();

				for (int i = 0; i < text.size(); i++) {
					((FFrameText) fframes.get(ft)).addText((String) text.get(i));
				}
			}
		}
	}

	/**
	 * Inicia la creacin de todas las platillas posibles a partir de un
	 * FFrameView.
	 *
	 * @param fframeview FFrameView a modificar.
	 *
	 * @throws DriverException
	 * @throws com.hardcode.gdbms.engine.data.DriverException
	 * @throws DriverIOException
	 */
	private void initTemplates(FFrameView fframeview)
		throws DriverException, com.hardcode.gdbms.engine.data.DriverException, 
			DriverIOException {
		//Inicio la opcin de cancelar la impresin de todas las fichas a true.
		printAll[1] = true;

		//Rectngulo del extent inicial de la vista contenida en el FFrameView.
		Rectangle2D.Double rectIni = new Rectangle2D.Double();
		rectIni.setRect(fframeview.getFMap().getViewPort().getExtent());

		//Entero que representa el tipo de escala que tiene seleccionada el FFrameView.
		int typeScaleIni = fframeview.getTypeScale();

		//Guarda los textos iniciales de todos los FFrameText
		initFFrameText();

		//Recorro todas las capas de la vista.
		for (int i = 0; i < fframeview.getFMap().getLayers().getLayersCount();
				i++) {
			//Compruebo que la capa este activada.
			if (fframeview.getFMap().getLayers().getLayer(i).isActive()) {
				//Obtengo su recordSet
				if (fframeview.getFMap().getLayers().getLayer(i) instanceof AlphanumericData) {
					SelectableDataSource dataSource = ((AlphanumericData) fframeview.getFMap()
																					.getLayers()
																					.getLayer(i)).getRecordset();

					//Recorro todos los registros.
					for (long k = 0; k < dataSource.getRowCount(); k++) {
						//Compruebo que est seleccionado.
						if (dataSource.isSelected((int) k)) {
							//Actualizo la posicin del puntero del recordSet.
							////dataSource.moveTo(k);
							//Obtengo el rectngulo del shape para despus ponerlo como extent del FFrameView
							Rectangle2D rec = ((SingleLayer) fframeview
											   .getFMap().getLayers().getLayer(i)).getSource()
											   .getShape((int) k).getBounds2D();

							//fframeview.setLinked(false);
							//Cambio el extent de la vista que contiene fframeview, 
							//del que tena al extent del shape seleccionado.
							fframeview.setNewExtent(rec);

							//Cambio el tipo de escala de fframeview a Escala especificada por el usuario.
							fframeview.setTypeScale(FFrameView.MANUAL);

							//Recorro todos los FFrames que contiene el Layout y compruebo que son una instancia de FFrameText, 
							//para inicializarlo con el valor correspondiente.
							for (int ft = 0; ft < fframes.size(); ft++) {
								if (fframes.get(ft) instanceof FFrameText) {
									if (fframeview != null) {
										initFFrameText((FFrameText) fframes.get(
												ft), dataSource, k);
									}
								}
							}

							if (isFirst) {
								//Si es el primer registro seleccionado se abre el dilogo 
								//para preguntar si se quiere imprimir todas las fichas.
								askAll();
								isFirst = false;
							}

							if (!printAll[1]) {
								//Si se cancela la impresin desde el dilogo askAll se termina la creacin de fichas.
								//Pongo isFirst a true para que la prxima vez que se llame 
								//a este mtodo se pregunte si se quiere imprimir todas las fichas.
								isFirst = true;

								return;
							}

							if (printAll[0]) {
								//Imprime la ficha actual.
								printExtension.printLayout(layout);
							} else {
								//Refrescar el Layout para ver la ficha actual en pantalla.
								layout.refresh();
								layout.refresh();

								//Preguntar si se quiere imprimir la ficha actual.
								askOne();
							}
						}
					}
				}
			}
		}

		//Recupera el Extent inicial del FFrameView.
		fframeview.getFMap().getViewPort().setExtent(rectIni);

		//Recupera el tipo de escala que tena seleccionada el FFrameView.
		fframeview.setTypeScale(typeScaleIni);

		//Recupera el texto de todos los FFrameText del Layout.
		endFFrameText();

		//Refresca el Layout en pantalla.
		layout.refresh();

		//Volvemos a inicializar el boolean que sirve para saber si estamos en la primera ficha a imprimir.
		isFirst = true;
	}

	/**
	 * Modifica el FFrameText que se le pasa como parmetro cambiando su texto
	 * por el valor del campo que representa el tag.
	 *
	 * @param fframetext FFrameText a modificar.
	 * @param dataSource DataSource para extraer el valor.
	 * @param k indice del registro.
	 *
	 * @throws com.hardcode.gdbms.engine.data.DriverException
	 */
	private void initFFrameText(FFrameText fframetext,
		SelectableDataSource dataSource, long k)
		throws com.hardcode.gdbms.engine.data.DriverException {
		//Recorro todos los campos.
		for (int j = 0; j < dataSource.getFieldCount(); j++) {
			//Compruebo si el tag es igual al campo, y si es as, 
			//borro el texto que contiene y le aado el valor.
			if (dataSource.getFieldName(j).equals(fframetext.getTag())) {
				fframetext.getText().clear();
				dataSource.start();
				fframetext.addText(dataSource.getDriver().getFieldValue(k, j)
											 .toString());
				dataSource.stop();
			}
		}
	}

	/**
	 * Abre el dilogo que pregunta si se quieren imprimir todas las fichas que
	 * se generan.
	 */
	public void askAll() {
		AskAll all = new AskAll(printAll);

		//Aade el objeto all que extiende JPanel 
		//e implementa com.iver.mdiApp.ui.MDIManager.View como una 
		//ventana ms de la aplicacin.
		PluginServices.getMDIManager().addView(all);
	}

	/**
	 * Abre el dilogo que pregunta si la ficha que se esta visualizando se
	 * quiere imprimir.
	 */
	public void askOne() {
		AskOne one = new AskOne(layout);

		//Aade el objeto one que extiende JPanel 
		//e implementa com.iver.mdiApp.ui.MDIManager.View como una 
		//ventana ms de la aplicacin.
		PluginServices.getMDIManager().addView(one);
	}

	/**
	 * @see com.iver.andami.plugins.Extension#inicializar()
	 */
	public void inicializar() {
	}

	/**
	 * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		//Obtengo el Layout que contiene la vista actualmente seleccionada en ANDAMI.
		layout = (Layout) PluginServices.getMDIManager().getActiveView();

		//Obtengo todos los FFrames que contiene el Layout.
		fframes = layout.getFFrames();

		//Recorro todos los FFrames comprobando si es una instacia de FFrameView 
		//para ejecutar initTemplates en el caso de que as sea.
		for (int i = 0; i < fframes.size(); i++) {
			if (fframes.get(i) instanceof FFrameView) {
				try {
					//Inicia la creacin de todas las platillas posibles a partir de fframeview.
					initTemplates((FFrameView) fframes.get(i));
				} catch (DriverException e) {
					e.printStackTrace();
				} catch (com.hardcode.gdbms.engine.data.DriverException e) {
					e.printStackTrace();
				} catch (DriverIOException e) {
					e.printStackTrace();
				}
			}
		}
	}
}
